@@ -1,8 +1,32 @@
 fdiscd.predict <-
-function(x, classe, crit = 1, gaussiand = TRUE, kern = NULL, windowh = NULL, misclass.ratio = FALSE)  
+function(xf, class.var, crit = 1, gaussiand = TRUE, kern = NULL, windowh = NULL, misclass.ratio = FALSE)
 {
+  # xf:     object of class 'folderh' with 2 data frames.
+  # class.var: character. Name of the column of xf[[1]] containing the class variable.
+                               
+  name.foldh <- as.character(match.call())[2]
+  name.x <- names(xf)[2]
+  name.classe <- names(xf)[1]
+  
+  if (!is.folderh(xf))
+    stop("fdiscd.predict applies to a folderh.\nNotice that for versions earlier than 2.0, fdiscd.predict applied to two data frames.")
+  if (length(xf) != 2)
+    stop("fdiscd.predict applies to a folderh with only two data frames.")
+  if (ncol(xf[[2]]) < 2)
+    stop(paste0(name.foldh, "[[", name.classe, "]] must have at least two columns (the grouping variable and the classes)."))
+  
+  x <- xf[[2]]
+  j.group <- which(colnames(x) == attr(xf, "keys"))
+  if (j.group != ncol(x))
+    x <- data.frame(x[-j.group], x[j.group])
+  classe <- xf[[1]][c(attr(xf, "keys"), class.var)]
+  
   # Number of variables:
   p <- ncol(x)-1
+  
+  if (!prod(apply(as.data.frame(x[,1:p]), 2, is.numeric)))
+    stop(paste0("Non numeric variable(s) in ", name.foldh, "$", name.x, "\n",
+                "Only the grouping variable should be a factor. The other variables must be numeric."))
 
   # If necessary: change the last variable of x (the groups) into a factor:
   if (!is.factor(x[, p+1]))  x[, p+1] <- as.factor(x[, p+1])
@@ -120,22 +144,22 @@ if (crit == 1)  {
     # Case: multivariate, gaussian distribution and estimated parameters
     mg.. =
         {for (i in pred.name)
-           {Wf[i] <- l2d.gp(moy.x[[i]],var.x[[i]],moy.x[[i]],var.x[[i]])
+           {Wf[i] <- l2dpar(moy.x[[i]],var.x[[i]],moy.x[[i]],var.x[[i]])
             for (j in classe.name)
-              Wfg[i,j]<-l2d.gp(moy.x[[i]],var.x[[i]],moy.cl[[j]],var.cl[[j]])
+              Wfg[i,j]<-l2dpar(moy.x[[i]],var.x[[i]],moy.cl[[j]],var.cl[[j]])
            }
          for (j in classe.name)
-            Wg[j] <- l2d.gp(moy.cl[[j]],var.cl[[j]],moy.cl[[j]],var.cl[[j]])
+            Wg[j] <- l2dpar(moy.cl[[j]],var.cl[[j]],moy.cl[[j]],var.cl[[j]])
          },
     # Case univariate, gaussian distributions with parametres internally estimed 
     ug.. =  
         {for (i in pred.name)
-           {Wf[i] <- l2d.gp.u(moy.x[[i]],var.x[[i]],moy.x[[i]],var.x[[i]])
+           {Wf[i] <- l2dpar(moy.x[[i]],var.x[[i]],moy.x[[i]],var.x[[i]])
             for (j in classe.name)
-              Wfg[i,j]<-l2d.gp.u(moy.x[[i]],var.x[[i]],moy.cl[[j]],var.cl[[j]])
+              Wfg[i,j]<-l2dpar(moy.x[[i]],var.x[[i]],moy.cl[[j]],var.cl[[j]])
            }
          for (j in classe.name)
-            Wg[j] <- l2d.gp.u(moy.cl[[j]],var.cl[[j]],moy.cl[[j]],var.cl[[j]])
+            Wg[j] <- l2dpar(moy.cl[[j]],var.cl[[j]],moy.cl[[j]],var.cl[[j]])
          },
     # Case: multivariate, non Gaussian distribution, density estimated using 
     # Gaussian kernel and AMISE window 
@@ -155,7 +179,7 @@ if (crit == 1)  {
           {n.x <- pred.name[i]
           ind.i <- which(x.pred[, p+1]==n.x)
           xi<-x.pred[ind.i,1:p]
-          Wf[n.x] <- l2d.kgw(xi,varLwL.f[[i]],xi,varLwL.f[[i]])
+          Wf[n.x] <- l2d(xi, xi, method="kern", varw1=varLwL.f[[i]], varw2=varLwL.f[[i]])
           for(j in 1:nb.classe)
            {n.cl <- classe.name[j]
            #Test if the determinant of 
@@ -164,14 +188,14 @@ if (crit == 1)  {
               stop("The matrices are not invertible")
             ind.j <- which(x[, p+2]==n.cl)
             xj<-x[ind.j,1:p]
-            Wfg[n.x, n.cl]<-l2d.kgw(xi,varLwL.f[[i]],xj,varLwL.g[[j]])
+            Wfg[n.x, n.cl]<-l2d(xi, xj, method="kern", varw1=varLwL.f[[i]], varw2=varLwL.g[[j]])
            }
           }
         for(j in 1:nb.classe)
           {n.cl <- classe.name[j]
            ind.j <- which(x[, p+2]==n.cl)
            xj<-x[ind.j,1:p]
-           Wg[n.cl] <- l2d.kgw(xj,varLwL.g[[j]],xj,varLwL.g[[j]])}
+           Wg[n.cl] <- l2d(xj, xj, method="kern", varw1=varLwL.g[[j]], varw2=varLwL.g[[j]])}
         },
     # Case univariate, non gaussian distributions estimated by gaussian kernel
     # method, and AMISE windows 
@@ -191,7 +215,7 @@ if (crit == 1)  {
           {n.x <- pred.name[i]
           ind.i <- which(x.pred[, p+1]==n.x)
           xi<-x.pred[ind.i,1:p]
-          Wf[n.x] <- l2d.kgw.u(xi,varLwL.f[[i]],xi,varLwL.f[[i]])
+          Wf[n.x] <- l2d(xi, xi, method="kern", varw1=varLwL.f[[i]], varw2=varLwL.f[[i]])
           for(j in 1:nb.classe)
             {n.cl <- classe.name[j]
             # Test if the variances are different from zero
@@ -200,14 +224,14 @@ if (crit == 1)  {
               }
             ind.j <- which(x[, p+2]==n.cl)
             xj<-x[ind.j,1:p]
-            Wfg[n.x,n.cl]<-l2d.kgw.u(xi,varLwL.f[[i]],xj,varLwL.g[[j]])
+            Wfg[n.x,n.cl]<-l2d(xi, xj, method="kern", varw1=varLwL.f[[i]], varw2=varLwL.g[[j]])
             }
           };
         for(j in 1:nb.classe)
           {n.cl <- classe.name[j]
           ind.j <- which(x[, p+2]==n.cl)
           xj<-x[ind.j,1:p]
-          Wg[n.cl] <- l2d.kgw.u(xj,varLwL.g[[j]],xj,varLwL.g[[j]])}
+          Wg[n.cl] <- l2d(xj, xj, method="kern", varw1=varLwL.g[[j]], varw2=varLwL.g[[j]])}
         },
     # Case: multivariate, non gaussian distributions estimed by gaussian kernel
     # method, and bandwith parameter, common to all densities, given by the user
@@ -225,7 +249,7 @@ if (crit == 1)  {
           {n.x <- pred.name[i]
            ind.i <- which(x.pred[, p+1]==n.x)
            xi<-x.pred[ind.i,1:p]
-           Wf[n.x] <- l2d.kgw(xi,varLwL.f[[i]],xi,varLwL.f[[i]])
+           Wf[n.x] <- l2d(xi, xi, method="kern", varw1=varLwL.f[[i]], varw2=varLwL.f[[i]])
            for(j in 1:nb.classe)
            {n.cl <- classe.name[j]
             # Test if the determinant of
@@ -234,14 +258,14 @@ if (crit == 1)  {
               stop("The matrices are not invertible")
             ind.j <- which(x[, p+2]==n.cl)
             xj<-x[ind.j,1:p]
-            Wfg[n.x,n.cl]<-l2d.kgw(xi,varLwL.f[[i]],xj,varLwL.g[[j]])
+            Wfg[n.x,n.cl]<-l2d(xi, xj, method="kern", varw1=varLwL.f[[i]], varw2=varLwL.g[[j]])
            }
           }
           for(j in 1:nb.classe)
            {n.cl <- classe.name[j]
             ind.j <- which(x[, p+2]==n.cl)
             xj<-x[ind.j,1:p]
-            Wg[n.cl] <- l2d.kgw(xj,varLwL.g[[j]],xj,varLwL.g[[j]])}
+            Wg[n.cl] <- l2d(xj, xj, method="kern", varw1=varLwL.g[[j]], varw2=varLwL.g[[j]])}
         },
     # Case univariate, non gaussian distributions estimed by gaussian kernel
     # method, and bandwith parameter, common to all densities, given by the user    
@@ -259,7 +283,7 @@ if (crit == 1)  {
           {n.x <- pred.name[i]
            ind.i <- which(x.pred[, p+1]==n.x)
            xi<-x.pred[ind.i,1:p]
-           Wf[n.x] <- l2d.kgw.u(xi,varLwL.f[[i]],xi,varLwL.f[[i]])
+           Wf[n.x] <- l2d(xi, xi, method="kern", varw1=varLwL.f[[i]], varw2=varLwL.f[[i]])
           for(j in 1:nb.classe)
            {n.cl <- classe.name[j]
             # Test if the variances are different from zero
@@ -267,14 +291,14 @@ if (crit == 1)  {
               stop("The matrices are not invertible")
             ind.j <- which(x[, p+2]==n.cl)
             xj<-x[ind.j,1:p]
-            Wfg[n.x,n.cl]<-l2d.kgw.u(xi,varLwL.f[[i]],xj,varLwL.g[[j]])
+            Wfg[n.x,n.cl]<-l2d(xi, xj, method="kern", varw1=varLwL.f[[i]], varw2=varLwL.g[[j]])
            }
           }
           for(j in 1:nb.classe)
            {n.cl <- classe.name[j]
             ind.j <- which(x[, p+2]==n.cl)
             xj<-x[ind.j,1:p]
-            Wg[n.cl] <- l2d.kgw.u(xj,varLwL.g[[j]],xj,varLwL.g[[j]])}
+            Wg[n.cl] <- l2d(xj, xj, method="kern", varw1=varLwL.g[[j]], varw2=varLwL.g[[j]])}
         }
     )
 }
@@ -292,7 +316,7 @@ if (crit %in% 2:3)
               if (abs(det(var.x[[i]]+var.x[[j]])) < .Machine$double.eps)
                 {stop(paste(c("Singular covariance matrix (group ", unique(names(var.x)[c(i, j)]), ")"), collapse = ""))
                 }
-                W[i,j]<-l2d.gp(moy.x[[i]],var.x[[i]],moy.x[[j]],var.x[[j]])
+                W[i,j]<-l2dpar(moy.x[[i]],var.x[[i]],moy.x[[j]],var.x[[j]])
               }
            }
          },
@@ -304,7 +328,7 @@ if (crit %in% 2:3)
             if (abs(var.x[[i]]+var.x[[j]]) < .Machine$double.eps)
               {stop(paste(c("Singular covariance matrix (group ", unique(names(var.x)[c(i, j)]), ")"), collapse = ""))
               }
-            W[i,j]<-l2d.gp.u(moy.x[[i]],var.x[[i]],moy.x[[j]],var.x[[j]])
+            W[i,j]<-l2dpar(moy.x[[i]],var.x[[i]],moy.x[[j]],var.x[[j]])
             }
           }
         },
@@ -327,7 +351,7 @@ if (crit %in% 2:3)
               stop("The matrices are not invertible")
             ind.j <- which(x[, p+1]==group.name[j])
             xj<-x[ind.j,1:p]
-            W[i,j]<-l2d.kgw(xi,varLwL[[i]],xj,varLwL[[j]])
+            W[i,j]<-l2d(xi, xj, method="kern", varw1=varLwL[[i]], varw2=varLwL[[j]])
            }
           }
         },
@@ -351,7 +375,7 @@ if (crit %in% 2:3)
               }
             ind.j <- which(x[, p+1]==group.name[j])
             xj<-x[ind.j,1:p]
-            W[i,j]<-l2d.kgw.u(xi,varLwL[[i]],xj,varLwL[[j]])
+            W[i,j]<-l2d(xi, xj, method="kern", varw1=varLwL[[i]], varw2=varLwL[[j]])
             }
           };
         },
@@ -369,7 +393,7 @@ if (crit %in% 2:3)
           for(j in 1:i)
            {ind.j <- which(x[, p+1]==group.name[j])
            xj<-x[ind.j,1:p]
-           W[i,j]<-l2d.kgw(xi,varLwL[[i]],xj,varLwL[[j]])
+           W[i,j]<-l2d(xi, xj, method="kern", varw1=varLwL[[i]], varw2=varLwL[[j]])
            }
           }
         },
@@ -387,7 +411,7 @@ if (crit %in% 2:3)
           for(j in 1:i)
            {ind.j <- which(x[, p+1]==group.name[j])
             xj<-x[ind.j,1:p]
-            W[i,j]<-l2d.kgw.u(xi,varLwL[[i]],xj,varLwL[[j]]) }};
+            W[i,j]<-l2d(xi, xj, method="kern", varw1=varLwL[[i]], varw2=varLwL[[j]]) }};
         }
     )
     for (i in 1:(nb.groups-1))
